package ece476controlmapper;

import java.awt.event.KeyEvent;
import java.util.ArrayList;


public class GestureControl implements Comparable<GestureControl> {
	
	public static int strokeTime = 400;
	public static int longStrokeTime = 1000;
	
	public static final int KEY_STROKE = 0;
	public static final int KEY_PRESS = 1;
	public static final int KEY_RELEASE = 2;
	public static final int KEY_LONG_STROKE = 3;
	
	private String name;
	private String id;
	private ArrayList<Integer> keycode;
	private int actionType;
	private boolean enabled;
	
	private int index;
	
	private static int nextIndex = 0;
	
	public GestureControl(String id, String name, int keycode) {
		this(id, name, KEY_STROKE, true);
		this.addKey(keycode);
	}
	
	public GestureControl(String id, String name, int actType, boolean enabled) {
		this.id = id;
		this.name = name;
		this.keycode = new ArrayList<Integer>(2);
		this.actionType = actType;
		this.enabled = enabled;
		this.index = nextIndex++;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public ArrayList<Integer> getKey() {
		return keycode;
	}

	public void setKey(int key) {
		this.keycode.clear();
		this.keycode.add(key);
	}
	
	public void addKey(int key) {
		this.keycode.add(key);
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	public int getActionType() {
		return actionType;
	}

	public void setActionType(int actionType) {
		this.actionType = actionType;
	}
	 
	public static GestureControl fromString(String s) {
		try {
			String[] tokens = s.split(",");
			
			if (tokens.length < 5)
				return null;
			
			for (int i = 0; i < tokens.length; i++) {
				tokens[i] = tokens[i].trim();
				if (tokens[i].length() == 0)
					return null;
			}
			GestureControl g = new GestureControl(
					tokens[0].trim(),                        // id
					tokens[1].trim(),                        // name
					Integer.parseInt(tokens[3].trim()),      // action type
					Boolean.parseBoolean(tokens[4].trim())); // enabled
			String keycodeString = tokens[2].trim();
			if (!keycodeString.startsWith("{")) {
				g.setKey(Integer.parseInt(keycodeString));
			}
			else {
				keycodeString = keycodeString.replace("{", "").replace("}", "").trim();
				String[] codes = keycodeString.split(" ");
				for (String code:codes) {
					g.addKey(Integer.parseInt(code.trim()));
				}
			}
			return g;
			
		} catch (Exception e) {
			return null;
		}
	}
	
	public static String toString(GestureControl g) {
		String s = g.id + ", " + g.name + ", { ";
		for (Integer i : g.keycode) {
			s += i + " ";
		}
		s += "}, " + g.actionType + ", " + g.enabled;
		return s;
	}

	@Override
	public int compareTo(GestureControl o) {
		return index - o.index;
	}

	public String getKeyText() {
		String s = "";
		for (int i = 0; i < keycode.size()-1; i++) {
			s += KeyEvent.getKeyText(keycode.get(i)) + "+";
		}
		s += KeyEvent.getKeyText(keycode.get(keycode.size()-1));
		return s;
	}

}
